unit Embedded;

// Copyright  1997 by Jon Shemitz, all rights reserved.
// Permission is hereby granted to freely use, modify, and
// distribute this source code PROVIDED that all six lines of
// this copyright and contact notice are included without any
// changes. Questions? Comments? Offers of work?
// mailto:jon@midnightbeach.com

// ------------------------------------------------------------

// The superclass for all forms-in-forms; adds constructor that
// sets Parent and positioning properly. Also adds support for
// "spans", to allow a frame to calculate the smallest rectangle
// that will hold all its views.

interface

uses
  Windows, Classes, Controls, Forms, InterfacedForms;

type
  EmbeddedFormMode = (efmZoomed, efmTopLeft, efmCentered);
  TSpan            = TPoint;  // A [Width, Height] pair

  TEmbeddedForm = class(TInterfacedForm)
  public
    constructor CreateEmbedded( _Owner: TComponent;
                                Frame:  TWinControl;
                                Mode:   EmbeddedFormMode );
    function Span: TSpan;
    function SpanEx(ExtraX, ExtraY: integer): TSpan;
  end;

function UnionSpan(A, B: TSpan): TSpan; // See UnionRect dox.

implementation

{$R *.DFM}

function ALZ(Number: integer): Cardinal; // At Least Zero
begin
  if Number > 0
    then Result := Number
    else Result := 0;
end;

constructor TEmbeddedForm.CreateEmbedded(
  _Owner: TComponent;
  Frame:  TWinControl;
  Mode:   EmbeddedFormMode );
begin
  Inherited Create(_Owner);

  Parent      := Frame;
  BorderIcons := [];
  BorderStyle := bsNone;

  case Mode of
    efmZoomed:   Align := alClient;
    efmTopLeft:  begin
                 Top  := 0;
                 Left := 0;
                 end; // efmTopLeft
    efmCentered: begin
                 Top  := ALZ((Frame.Height - Height) div 2);
                 Left := ALZ((Frame.Width  - Width)  div 2);
                 end; // efmCentered
    else         Assert(False);
    end; // case
  Visible := True;
end; // TEmbeddedForm.CreateEmbedded

function TEmbeddedForm.Span: TSpan;
begin
  Result.X := Width;
  Result.Y := Height;
end; // TEmbeddedForm.Span

function TEmbeddedForm.SpanEx(ExtraX, ExtraY: integer): TSpan;
begin
  Result.X := Width  + ExtraX;
  Result.Y := Height + ExtraY;
end; // TEmbeddedForm.SpanEx

function UnionSpan(A, B: TSpan): TSpan;
begin
  if A.X > B.X then Result.X := A.X else Result.X := B.X;
  if A.Y > B.Y then Result.Y := A.Y else Result.Y := B.Y;
end; // UnionSpan

end.
