unit FickleViews;

// Copyright  1997 by Jon Shemitz, all rights reserved.
// Permission is hereby granted to freely use, modify, and
// distribute this source code PROVIDED that all six lines of
// this copyright and contact notice are included without any
// changes. Questions? Comments? Offers of work?
// mailto:jon@midnightbeach.com

// ------------------------------------------------------------

// An FickleView is a View that may hold incomplete or
// inconsistent data that should NOT be written back to the
// Model object - a missing required field, a stop date before a
// start date, &c. When Valid is False, Wizards will not let
// the user move to the next page; Property Sheets will disable
// the OK button.

interface

uses
  Models, Views;

type
  TFickleView = class(TAbstractView)
  private
    { Private declarations }
    fValid:  boolean;
    fNotify: IFrame;
    // This implementation of IValid only supports 1 Notifiee
  public
    { Public declarations }
    procedure AddNotifiee(   Notify: IFrame); override;
    procedure RemoveNotifiee(Notify: IFrame); override;

    function  GetValid: boolean;              override;
    procedure SetValid(Value: boolean);       override;
  end;

implementation

{$R *.DFM}

procedure TFickleView.AddNotifiee(Notify: IFrame);
begin
  fNotify := Notify;
end; // TFickleView.AddNotifiee

procedure TFickleView.RemoveNotifiee(Notify: IFrame);
begin
  fNotify := Nil;
end; // TFickleView.RemoveNotifiee

function  TFickleView.GetValid: boolean;
begin
  Result := fValid;
end; // TFickleView.GetValid

procedure TFickleView.SetValid(Value: boolean);
begin
  if Value <> fValid then
    begin
    fValid := Value;
    if Assigned(fNotify) then
      fNotify.OnValidChanged(Self, Self);
    end; // Value <> fValid
end; // TFickleView.SetValid

end.

