unit ILists;

// A TList class that treats its entries as IUnknown references.

interface

uses Classes;

type
  TInterfaceList = array[0..MaxListSize - 1] of IUnknown;
  PInterfaceList = ^ TInterfaceList;

  TIList = class (TList)
           private
             function GetItem(N: integer): IUnknown;
             procedure SetItem(N: integer; Value: IUnknown);
             function GetList: PInterfaceList;
           public
             destructor Destroy; override;
             function Add(Item: IUnknown): Integer;
             procedure Clear;
             procedure Delete(Index: Integer);
             function First: IUnknown;
             function IndexOf(Item: IUnknown): Integer;
             procedure Insert(Index: Integer; Item: IUnknown);
             function Last: IUnknown;
             function Remove(Item: IUnknown): Integer;
             property Items[Index: Integer]: IUnknown
                      read GetItem write SetItem; default;
             property List: PInterfaceList read GetList;
           end;

implementation

function TIList.GetItem(N: integer): IUnknown;
begin
  Result := IUnknown(inherited Items[N])
end; // TIList.GetItem

procedure TIList.SetItem(N: integer; Value: IUnknown);
begin
  List^[N] := Value;
end; // TIList.SetItem

function TIList.GetList: PInterfaceList;
begin
  Result := PInterfaceList(inherited List);
end; // TIList.GetList

destructor TIList.Destroy;
begin
  Clear;
  inherited Destroy;
end; // TIList.Destroy

function TIList.Add(Item: IUnknown): Integer;
begin
  Result := inherited Add(Nil);
  Items[Result] := Item;
end; // TIList.Add

procedure TIList.Clear;
var
  Index: integer;
begin
  for Index := 0 to Count - 1 do // release the reference
    Items[Index] := Nil;
  inherited Clear;
end; // TIList.Clear

procedure TIList.Delete(Index: Integer);
begin
  Items[Index] := Nil;
  inherited Delete(Index);
end; // TIList.Delete

function TIList.First: IUnknown;
begin
  Result := IUnknown(inherited First);
end; // TIList.First

function TIList.IndexOf(Item: IUnknown): Integer;
begin
  Result := inherited IndexOf(pointer(Item));
end; // TIList.IndexOf

procedure TIList.Insert(Index: Integer; Item: IUnknown);
begin
  inherited Insert(Index, Nil);
  Items[Index] := Item;
end; // TIList.Insert

function TIList.Last: IUnknown;
begin
  Result := IUnknown(inherited Last);
end; // TIList.Last

function TIList.Remove(Item: IUnknown): Integer;
begin
  Result := IndexOf(Item);
  Delete(Result);
end; // TIList.Remove

end.

