unit InterfacedForms;

// Copyright  1997 by Jon Shemitz, all rights reserved.
// Permission is hereby granted to freely use, modify, and
// distribute this source code PROVIDED that all six lines of
// this copyright and contact notice are included without any
// changes. Questions? Comments? Offers of work?
// mailto:jon@midnightbeach.com

// ------------------------------------------------------------

// Adds a functional IUnknown implementation to TForm. 

interface

uses Classes, Forms;

type
  TInterfacedForm =
    class (TForm, IUnknown)
    private
      fRefCount: integer;
    protected
      function QueryInterface( const IID: TGUID;
                               out Obj): Integer; stdcall;
      function _AddRef: Integer;                  stdcall;
      function _Release: Integer;                 stdcall;
    public
      property RefCount: integer read fRefCount write fRefCount;
    end;

implementation

uses Windows; // for E_NOINTERFACE

// IUnknown code based on the TInterfacedObject source

function TInterfacedForm.QueryInterface( const IID: TGUID;
                                         out Obj): Integer;
begin
  if GetInterface(IID, Obj)
    then Result := 0
    else Result := E_NOINTERFACE;
end;

function TInterfacedForm._AddRef: Integer;
begin
  Inc(fRefCount);
  Result := fRefCount;
end;

function TInterfacedForm._Release: Integer;
begin
  Dec(fRefCount);
  Result := fRefCount;
  if fRefCount = 0 then Destroy;
end;

end.

