unit ModelEditors;

// Copyright  1997 by Jon Shemitz, all rights reserved.
// Permission is hereby granted to freely use, modify, and
// distribute this source code PROVIDED that all six lines of
// this copyright and contact notice are included without any
// changes. Questions? Comments? Offers of work?
// mailto:jon@midnightbeach.com

// ------------------------------------------------------------

// This is the superclass for TAbstractWizard and
// TAbstractPropertySheet; it implements a generic EditModel.

interface

uses Windows, Models, InterfacedForms;

type
  TModelEditor =
    class (TInterfacedForm, IModelEdit)
    private
      fReadOnly: boolean;

    protected
      function  GetReadOnly: boolean;        virtual;
      procedure SetReadOnly(Value: boolean); virtual;

      procedure Initialize;                  virtual; abstract;
      function RunEditor(Model: TModel):
                                boolean;     virtual; abstract;
      procedure Finalize;                    virtual; abstract;

      function EditModel(Model: TModel): boolean;
    public
      property ReadOnly: boolean read  fReadOnly
                                 write SetReadOnly;
    end;

implementation

function  TModelEditor.GetReadOnly: boolean;
begin
  Result := fReadOnly;
end; // TModelEditor.GetReadOnly

procedure TModelEditor.SetReadOnly(Value: boolean);
begin
  fReadOnly := Value;
end; // TModelEditor.SetReadOnly

function TModelEditor.EditModel(Model: TModel): boolean;
begin
  Initialize;
  try
    ReadOnly := ReadOnly; // set views
    Result := RunEditor(Model);
  finally Finalize; end;
end; // TModelEditor.EditModel

end.

