unit Models;

// Copyright  1997 by Jon Shemitz, all rights reserved.
// Permission is hereby granted to freely use, modify, and
// distribute this source code PROVIDED that all six lines of
// this copyright and contact notice are included without any
// changes. Questions? Comments? Offers of work?
// mailto:jon@midnightbeach.com

// ------------------------------------------------------------

// Basic declarations for the model/view/frame abstractions:

// A "model" is a representation of something important to your
// program. It's often declared in a datamodule, which exports
// procedures to create/save/load model objects.

// A "view" allows a user to see/manipulate (parts of) a model.

// A "frame" is a container control where a view may be placed.

interface

uses Graphics;

type
  TModel      = TObject;

  // Both IView and IModelEdit have a ReadOnly property
  IReadOnly =  interface
               function  GetReadOnly: boolean;
               procedure SetReadOnly(Value: boolean);
               property  ReadOnly: boolean read  GetReadOnly
                                           write SetReadOnly;
               end;

  // Fill a view from a model & write changes back;
  // frame/view interactions
  IFrame =     interface;
  IView =      interface (IReadOnly)
               procedure ReadFromModel(Model: TModel);
               procedure WriteToModel(Model: TModel);

               function  GetValid: boolean;
               procedure SetValid(Value: boolean);
               property  Valid: boolean read GetValid
                                        write SetValid;


               procedure AddNotifiee(   Notify: IFrame);
               procedure RemoveNotifiee(Notify: IFrame);
               end;
  IFrame =     interface
               procedure OnValidChanged(
                 ChangingObject: TObject;
                 View:           IView );
               end;

  // Wizards and Property Sheets are "model editors"
  IModelEdit = interface (IReadOnly)
               // Low level routines that allow an app to setup
               // an editor once, and run it several times.
               procedure Initialize;
               function RunEditor(Model: TModel): boolean;
               procedure Finalize;
               // Initialize/RunEditor/Finalize
               function EditModel(Model: TModel): boolean;
               end;

const
  ShowReadOnly_EditColors: array[boolean] of TColor = (clWindow, clBtnFace);

implementation

end.

