unit ValidViews;

// Copyright  1997 by Jon Shemitz, all rights reserved.
// Permission is hereby granted to freely use, modify, and
// distribute this source code PROVIDED that all six lines of
// this copyright and contact notice are included without any
// changes. Questions? Comments? Offers of work?
// mailto:jon@midnightbeach.com

// ------------------------------------------------------------

// A ValidView is a View that can never hold invalid data - like
// a Notes field. It is also useful for splash panels like
// "welcome to the wizard" or "all done"

interface

uses
  Models, Views;

type
  TValidView = class(TAbstractView)
  private
    { Private declarations }
  public
    { Public declarations }
    function  GetValid: boolean;              override;
    procedure SetValid(Value: boolean);       override;
    procedure AddNotifiee(   Notify: IFrame); override;
    procedure RemoveNotifiee(Notify: IFrame); override;
  end;

implementation

{$R *.DFM}

function TValidView.GetValid: boolean;
begin
  Result := True;
end; // TValidView.GetValid

procedure TValidView.SetValid(Value: boolean);
begin
  // A TValidView is always Valid - ignore the Value
end; // TValidView.SetValid

procedure TValidView.AddNotifiee(Notify: IFrame);
begin
  // A TValidView is always Valid - ignore the Add request
end; // TValidView.AddNotifiee

procedure TValidView.RemoveNotifiee(Notify: IFrame);
begin
  // A TValidView is always Valid - ignore the Remove request
end; // TValidView.RemoveNotifiee

end.

