unit Views;

// Copyright  1997 by Jon Shemitz, all rights reserved.
// Permission is hereby granted to freely use, modify, and
// distribute this source code PROVIDED that all six lines of
// this copyright and contact notice are included without any
// changes. Questions? Comments? Offers of work?
// mailto:jon@midnightbeach.com

// ------------------------------------------------------------

// Maps the IView contract onto an embedded form. Typically,
// you would derive views from TValidView or TFickleView.

interface

uses
  Models, Embedded;

type
  TAbstractView = class(TEmbeddedForm, IView)
    procedure FormCreate(Sender: TObject);
  private
    fReadOnly: boolean;
  protected
    function  GetValid: boolean;              virtual; abstract;
    procedure SetValid(Value: boolean);       virtual; abstract;

    function  GetReadOnly: boolean;           virtual;
    procedure SetReadOnly(Value: boolean);    virtual;
  public
    procedure ReadFromModel(Model: TModel);   virtual;
    procedure WriteToModel(Model:  TModel);   virtual;

    procedure AddNotifiee(   Notify: IFrame); virtual; abstract;
    procedure RemoveNotifiee(Notify: IFrame); virtual; abstract;

    property Valid:    boolean read GetValid write SetValid;
    property ReadOnly: boolean read  fReadOnly
                               write SetReadOnly;
  end;

  TViewClass = class of TAbstractView;

implementation

{$R *.DFM}

function  TAbstractView.GetReadOnly: boolean;
begin
  Result := fReadOnly;
end; // TAbstractView.GetReadOnly

procedure TAbstractView.SetReadOnly(Value: boolean);
begin
  fReadOnly := Value;
  Enabled   := not Value;
            // A read only view will display information but not
            // let users change it; you can override SetReadOnly
            // to change the appearance of read only views.
end; // TAbstractView.SetReadOnly

procedure TAbstractView.ReadFromModel(Model: TModel);
begin
end; // TAbstractView.ReadFromModel

procedure TAbstractView.WriteToModel(Model: TModel);
begin
end; // TAbstractView.WriteToModel

procedure TAbstractView.FormCreate(Sender: TObject);
begin
  inherited;
  _AddRef;  // so can pass Self as an interface reference
end;

end.

